@@ -1387,7 +1387,7 @@ sub paymentaccounts {
                  l.description AS translation
 		 FROM chart c
 		 LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
-		 WHERE c.link LIKE '%\\_paid'
+		 WHERE c.link LIKE '%_paid'
 		 ORDER BY c.accno|;
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
