@@ -24,6 +24,7 @@ sub invoice_details {
 
   my $query;
   my $sth;
+  my $ref;
 
   $form->{total} = 0;
 
@@ -32,9 +33,10 @@ sub invoice_details {
   # this is for the template
   $form->{invdate} = $form->{transdate};
   $form->{invdescription} = $form->{description};
+
+  for (qw(description projectnumber)) { delete $form->{$_} }
   
   my $tax;
-  my $item;
   my $i;
   my @sortlist = ();
   my $projectnumber;
@@ -158,8 +160,8 @@ sub invoice_details {
   $form->format_string(qw(text_packages));
   $form->format_amount($myconfig, $form->{packages});
   
-  $form->{projectnumber} = ();
-  $form->{description} = ();
+  @{ $form->{projectnumber} } = ();
+  @{ $form->{description} } = ();
  
   # sort the whole thing by project and group
   @sortlist = sort { $a->[5] cmp $b->[5] } @sortlist;
@@ -174,7 +176,7 @@ sub invoice_details {
   @{ $form->{lineitems} } = ();
   @{ $form->{taxrates} } = ();
 
-  foreach $item (@sortlist) {
+  for my $item (@sortlist) {
 
     $i = $item->[0];
     $j++;
@@ -204,12 +206,12 @@ sub invoice_details {
 	  }
     
 	  push(@{ $form->{description} }, $item->[2]);
-	  for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber bin qty ship unit deliverydate projectnumber sellprice listprice netprice discount discountrate linetotal itemnotes package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
+	  for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber bin qty ship unit deliverydate projectnumber sell sellprice listprice netprice discount discountrate linetotal itemnotes lineitemdetail package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
 	  push(@{ $form->{lineitems} }, { amount => 0, tax => 0 });
 	}
       }
     }
-      
+
     $form->{"qty_$i"} = $form->parse_amount($myconfig, $form->{"qty_$i"});
     
     if ($form->{"qty_$i"}) {
@@ -227,8 +229,8 @@ sub invoice_details {
       ($projectnumber) = split /--/, $form->{"projectnumber_$i"};
       push(@{ $form->{projectnumber} }, $projectnumber);
       
-      for (qw(sku serialnumber ordernumber customerponumber bin description unit deliverydate sellprice listprice package netweight grossweight volume countryorigin hscode barcode itemnotes)) { push(@{ $form->{$_} }, $form->{"${_}_$i"}) }
-	
+      for (qw(sku serialnumber ordernumber customerponumber bin description unit deliverydate sell sellprice listprice itemnotes lineitemdetail package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, $form->{"${_}_$i"}) }
+
       push(@{ $form->{qty} }, $form->format_amount($myconfig, $form->{"qty_$i"}));
       push(@{ $form->{ship} }, $form->format_amount($myconfig, $form->{"qty_$i"}));
 
@@ -286,7 +288,7 @@ sub invoice_details {
 	$taxamount = $linetotal * $taxrate / (1 + $taxrate);
 	$taxbase = ($linetotal - $taxamount);
 
-	foreach $item (@taxaccounts) {
+	for my $item (@taxaccounts) {
 	  if (($form->{"${item}_rate"} * $ml) > 0) {
 	    
 	    push @taxrates, $form->{"${item}_rate"} * 100;
@@ -336,7 +338,7 @@ sub invoice_details {
 	      push(@{ $form->{part} }, NULL);
 	    }
 
-	    for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber bin qty ship unit deliverydate projectnumber sellprice listprice netprice discount discountrate itemnotes package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
+	    for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber bin qty ship unit deliverydate projectnumber sell sellprice listprice netprice discount discountrate itemnotes lineitemdetail package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
 	    
 	    push(@{ $form->{description} }, $form->{groupsubtotaldescription});
 	    
@@ -363,7 +365,7 @@ sub invoice_details {
 	      push(@{ $form->{part} }, NULL);
 	    }
 
-	    for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber bin qty ship unit deliverydate projectnumber sellprice listprice netprice discount discountrate itemnotes package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
+	    for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber bin qty ship unit deliverydate projectnumber sell sellprice listprice netprice discount discountrate itemnotes lineitemdetail package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
 
 	    push(@{ $form->{description} }, $form->{groupsubtotaldescription});
 	    push(@{ $form->{linetotal} }, $form->format_amount($myconfig, $subtotal, $form->{precision}));
@@ -406,8 +408,8 @@ sub invoice_details {
     }
   }
   $i = 1;
-  for (@{ $form->{lineitems} }) {
-    push(@{ $form->{linetax} }, $form->format_amount($myconfig, $_->{tax}, $form->{precision}, ""));
+  for $ref (@{ $form->{lineitems} }) {
+    push(@{ $form->{linetax} }, $form->format_amount($myconfig, $ref->{tax}, $form->{precision}, ""));
   }
   
   
@@ -477,6 +479,16 @@ sub invoice_details {
 
   my ($paymentaccno) = split /--/, $form->{"AR_paid_$form->{paidaccounts}"};
   
+  $form->{roundto} = 0;
+  my %roundchange;
+  if ($form->{roundchange}) {
+    %roundchange = split /[=;]/, $form->unescape($form->{roundchange});
+    $form->{roundto} = $roundchange{''};
+    if ($form->{selectpaymentmethod}) {
+      $form->{roundto} = $roundchange{$form->{"paymentmethod_$form->{paidaccounts}"}};
+    }
+  }
+  
   for $i (1 .. $form->{paidaccounts}) {
     if ($form->{"paid_$i"}) {
       push(@{ $form->{payment} }, $form->{"paid_$i"});
@@ -485,13 +497,21 @@ sub invoice_details {
       push(@{ $form->{paymentdate} }, $form->{"datepaid_$i"});
       push(@{ $form->{paymentsource} }, $form->{"source_$i"});
       push(@{ $form->{paymentmemo} }, $form->{"memo_$i"});
-
+      
       ($description) = split /--/, $form->{"paymentmethod_$i"};
       push(@{ $form->{paymentmethod} }, $description);
+      
+      if ($form->{selectpaymentmethod}) {
+	$form->{roundto} = $roundchange{$form->{"paymentmethod_$i"}};
+      }
 
       $form->{paid} += $form->parse_amount($myconfig, $form->{"paid_$i"});
     }
   }
+  $form->{payment_method} = $form->{"paymentmethod_$form->{paidaccounts}"};
+  $form->{payment_method} =~ s/--.*//;
+  $form->{payment_accno} = $form->{"AR_paid_$form->{paidaccounts}"};
+  $form->{payment_accno} =~ s/--.*//;
 
   if ($form->{cdt} && $form->{discount_paid}) {
     $form->{invtotal} = $form->{cd_subtotal} + $cd_tax;
@@ -500,8 +520,6 @@ sub invoice_details {
 
   $form->{cd_invtotal} = $form->{cd_subtotal} + $cd_tax;
 
-  $form->{total} = $form->{invtotal} - $form->{paid};
-  
   if (!$form->{cd_amount}) {
     $form->{cd_available} = 0;
     $form->{cd_subtotal} = 0;
@@ -513,13 +531,20 @@ sub invoice_details {
 
   my $whole;
   my $decimal;
-  
+
   ($whole, $decimal) = split /\./, $form->{invtotal};
   $form->{decimal} = substr("${decimal}00", 0, 2);
   $form->{text_decimal} = $c->num2text($form->{decimal} * 1);
   $form->{text_amount} = $c->num2text($whole);
   $form->{integer_amount} = $whole;
 
+  if ($form->{roundto} > 0.01) {
+    $form->{total} = $form->round_amount($form->round_amount(($form->{invtotal} - $form->{paid}) / $form->{roundto}, 0) * $form->{roundto}, $form->{precision});
+    $form->{roundingdifference} = $form->round_amount($form->{paid} + $form->{total} - $form->{invtotal}, $form->{precision});
+  } else {
+    $form->{total} = $form->{invtotal} - $form->{paid};
+  }
+ 
   ($whole, $decimal) = split /\./, $form->{total};
   $form->{out_decimal} = substr("${decimal}00", 0, 2);
   $form->{text_out_decimal} = $c->num2text($form->{out_decimal} * 1);
@@ -573,8 +598,6 @@ sub delete_invoice {
     $dbh = $form->dbconnect_noauto($myconfig);
   }
 
-  $form->{id} *= 1;
-  
   &reverse_invoice($dbh, $form);
   
   my %audittrail = ( tablename  => 'ar',
@@ -620,7 +643,9 @@ sub delete_invoice {
 
   if ($rc) {
     foreach $spoolfile (@spoolfiles) {
-      unlink "$spool/$spoolfile" if $spoolfile;
+      if (-f "$spool/$myconfig->{dbname}/$spoolfile") {
+	unlink "$spool/$myconfig->{dbname}/$spoolfile";
+      }
     }
   }
   
@@ -631,6 +656,52 @@ sub delete_invoice {
 }
 
 
+sub delete_invoices {
+  my ($self, $myconfig, $form, $spool) = @_;
+  
+  $myconfig->{numberformat} = "1000.00";
+
+  # connect to database
+  my $dbh = $form->dbconnect($myconfig);
+
+  my $query;
+  my $where = "a.amount > 0 AND a.paid = 0";
+
+  if ($form->{transdatefrom}) {
+    $where .= " AND a.transdate >= '$form->{transdatefrom}'";
+  }
+  if ($form->{transdateto}) {
+    $where .= " AND a.transdate <= '$form->{transdateto}'";
+  }
+
+  $query = qq|SELECT a.id, a.invnumber
+              FROM ar a
+	      WHERE $where
+	      ORDER BY a.transdate|;
+  my $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+  
+  $form->{type} = "invoice";
+
+  $dbh->{AutoCommit} = 0;
+
+  while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
+
+    for (qw(id invnumber)) { $form->{$_} = $ref->{$_} }
+
+    print "$ref->{invnumber} ";
+
+    # reverse the invoice, keep exchangerate on file
+    IS->delete_invoice($myconfig, $form, $spool, $dbh);
+
+  }
+  $sth->finish;
+
+  $dbh->disconnect;
+
+}
+  
+
 sub assembly_details {
   my ($myconfig, $form, $dbh, $id, $qty) = @_;
   
@@ -641,7 +712,7 @@ sub assembly_details {
   if ($form->{format} eq 'html') {
     $spacer = "&nbsp;" x (3 * ($form->{stagger} - 1)) if $form->{stagger} > 1;
   }
-  if ($form->{format} =~ /(postscript|pdf)/) {
+  if ($form->{format} =~ /(ps|pdf)/) {
     if ($form->{stagger} > 1) {
       $spacer = ($form->{stagger} - 1) * 3;
       $spacer = '\rule{'.$spacer.'mm}{0mm}';
@@ -676,7 +747,7 @@ sub assembly_details {
     }
 
     if ($form->{grouppartsgroup} && $ref->{partsgroup} ne $sm) {
-      for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber unit qty ship bin deliverydate projectnumber sellprice listprice netprice discount discountrate linetotal itemnotes package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
+      for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber unit qty ship bin deliverydate projectnumber sellprice listprice netprice discount discountrate linetotal itemnotes lineitemdetail package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
       $sm = ($form->{"a_partsgroup"}) ? $form->{"a_partsgroup"} : "--";
       push(@{ $form->{description} }, "$spacer$sm");
       push(@{ $form->{lineitems} }, { amount => 0, tax => 0 });
@@ -685,7 +756,7 @@ sub assembly_details {
     if ($form->{stagger}) {
       
       push(@{ $form->{description} }, $form->format_amount($myconfig, $ref->{qty} * $form->{"qty_$i"}) . qq| -- $form->{"a_partnumber"}, $form->{"a_description"}|);
-      for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber unit qty ship bin deliverydate projectnumber sellprice listprice netprice discount discountrate linetotal itemnotes package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
+      for (qw(taxrates runningnumber number sku serialnumber ordernumber customerponumber unit qty ship bin deliverydate projectnumber sellprice listprice netprice discount discountrate linetotal itemnotes lineitemdetail package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
       
     } else {
       
@@ -693,7 +764,7 @@ sub assembly_details {
       push(@{ $form->{number} }, $form->{"a_partnumber"});
       push(@{ $form->{sku} }, $form->{"a_partnumber"});
 
-      for (qw(taxrates runningnumber ship serialnumber ordernumber customerponumber reqdate projectnumber sellprice listprice netprice discount discountrate linetotal itemnotes package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
+      for (qw(taxrates runningnumber ship serialnumber ordernumber customerponumber reqdate projectnumber sellprice listprice netprice discount discountrate linetotal itemnotes lineitemdetail package netweight grossweight volume countryorigin hscode barcode)) { push(@{ $form->{$_} }, "") }
       
     }
 
@@ -745,7 +816,7 @@ sub post_invoice {
   my $keepcleared;
   my $ok;
   
-  %$form->{acc_trans} = ();
+  %{ $form->{acc_trans} } = ();
 
   ($null, $form->{employee_id}) = split /--/, $form->{employee};
   unless ($form->{employee_id}) {
@@ -772,7 +843,7 @@ sub post_invoice {
 	      WHERE pt.parts_id = ?|;
   my $ptt = $dbh->prepare($query) || $form->dberror($query);
  
-  if ($form->{id} *= 1) {
+  if ($form->{id}) {
     $keepcleared = 1;
     $query = qq|SELECT id FROM ar
                 WHERE id = $form->{id}|;
@@ -811,11 +882,9 @@ sub post_invoice {
     $dbh->do($query) || $form->dberror($query);
   }
 
-  $form->{exchangerate} = $form->parse_amount($myconfig, $form->{exchangerate});
-  $form->{exchangerate} ||= 1;
+  $form->{exchangerate} = $form->parse_amount($myconfig, $form->{exchangerate}) || 1;
 
   my $i;
-  my $item;
   my $allocated = 0;
   my $taxrate;
   my $tax;
@@ -838,11 +907,11 @@ sub post_invoice {
 
   $form->{taxincluded} *= 1;
 
-  foreach my $i (1 .. $form->{rowcount}) {
+  for $i (1 .. $form->{rowcount}) {
     $form->{"qty_$i"} = $form->parse_amount($myconfig, $form->{"qty_$i"}) * $sw;
     
     if ($form->{"qty_$i"}) {
-      
+
       $pth->execute($form->{"id_$i"});
       $ref = $pth->fetchrow_hashref(NAME_lc);
       for (keys %$ref) { $form->{"${_}_$i"} = $ref->{$_} }
@@ -872,10 +941,13 @@ sub post_invoice {
       my $decimalplaces = ($dec > $form->{precision}) ? $dec : $form->{precision};
       
       # undo discount formatting
+      my $discount = $form->{"discount_$i"};
       $form->{"discount_$i"} = $form->parse_amount($myconfig, $form->{"discount_$i"})/100;
-     
+      
+      $discount = $form->round_amount($fxsellprice * $discount/100, $decimalplaces);
+      
       # deduct discount
-      $form->{"sellprice_$i"} = $fxsellprice - $form->round_amount($fxsellprice * $form->{"discount_$i"}, $decimalplaces);
+      $form->{"sellprice_$i"} = $fxsellprice - $discount;
       
       # linetotal
       my $fxlinetotal = $form->round_amount($form->{"sellprice_$i"} * $form->{"qty_$i"}, $form->{precision});
@@ -887,8 +959,7 @@ sub post_invoice {
       @taxaccounts = split / /, $form->{"taxaccounts_$i"};
       $ml = 1;
       $tax = 0;
-      $fxtax = 0;
-      
+
       for (0 .. 1) {
 	$taxrate = 0;
 
@@ -923,7 +994,7 @@ sub post_invoice {
 	$linetotal -= $form->round_amount($tax - $diff, $form->{precision});
 	$diff = ($amount - $tax);
       }
-      
+
       # add linetotal to income
       $amount = $form->round_amount($linetotal, $form->{precision});
 
@@ -1056,20 +1127,24 @@ sub post_invoice {
       $form->{datepaid} = $form->{"datepaid_$i"};
     }
   }
-  
+
   # add lineitems + tax
   $amount = 0;
   $grossamount = 0;
   $fxamount = 0;
-  for (@{ $form->{acc_trans}{lineitems} }) {
-    $amount += $_->{amount};
-    $grossamount += $_->{grossamount};
-    $fxamount += $_->{fxamount};
+  for $ref (@{ $form->{acc_trans}{lineitems} }) {
+    $amount += $ref->{amount};
+    $grossamount += $ref->{grossamount};
+    $fxamount += $ref->{fxamount};
   }
   $invnetamount = $amount;
 
   $amount = 0;
-  for (split / /, $form->{taxaccounts}) { $amount += $form->{acc_trans}{$form->{id}}{$_}{amount} = $form->round_amount($form->{acc_trans}{$form->{id}}{$_}{amount}, $form->{precision}) }
+  for (split / /, $form->{taxaccounts}) {
+    $form->{acc_trans}{$form->{id}}{$_}{amount} = $form->round_amount($form->{acc_trans}{$form->{id}}{$_}{amount}, 10);
+
+    $amount += $form->{acc_trans}{$form->{id}}{$_}{amount} = $form->round_amount($form->{acc_trans}{$form->{id}}{$_}{amount}, $form->{precision});
+  }
   $invamount = $invnetamount + $amount;
 
   $diff = 0;
@@ -1082,7 +1157,9 @@ sub post_invoice {
   $invnetamount += $fxdiff;
   $invamount += $fxdiff;
 
-  if ($form->round_amount($form->{paid} - $fxamount + $fxtax, $form->{precision}) == 0) {
+  $fxtax = $form->round_amount($fxtax, $form->{precision});
+
+  if ($form->round_amount($form->{paid} - ($fxamount + $fxtax), $form->{precision}) == 0) {
     $form->{paid} = $invamount;
   } else {
     $form->{paid} = $form->round_amount($form->{paid} * $form->{exchangerate}, $form->{precision});
@@ -1104,7 +1181,7 @@ sub post_invoice {
   delete $form->{acc_trans}{lineitems};
   
   # update exchangerate
-  $form->update_exchangerate($dbh, $form->{currency}, $form->{transdate}, $form->{exchangerate}, 0);
+  $form->update_exchangerate($dbh, $form->{currency}, $form->{transdate}, $form->{exchangerate});
 
   my $accno;
   my ($araccno) = split /--/, $form->{AR};
@@ -1130,8 +1207,10 @@ sub post_invoice {
     $tax = 0;
     $fxtax = 0;
     
-    $form->{"exchangerate_$i"} = $form->parse_amount($myconfig, $form->{"exchangerate_$i"});
-    $form->{"exchangerate_$i"} ||= 1;
+    $form->{"exchangerate_$i"} = $form->parse_amount($myconfig, $form->{"exchangerate_$i"}) || 1;
+   
+    # update exchangerate
+    $form->update_exchangerate($dbh, $form->{currency}, $form->{"datepaid_$i"}, $form->{"exchangerate_$i"});
 
     # calculate tax difference
     my $discount = 0;
@@ -1210,7 +1289,7 @@ sub post_invoice {
   if ($invamount == 0) {
     $form->{receivables} = 1;
   }
-  
+
   my $cleared = 'NULL';
   my $voucherid;
   my $approved;
@@ -1231,9 +1310,11 @@ sub post_invoice {
       $form->{"datepaid_$i"} = $form->{transdate} unless ($form->{"datepaid_$i"});
       $form->{datepaid} = $form->{"datepaid_$i"};
       
-      $form->{"exchangerate_$i"} = $form->parse_amount($myconfig, $form->{"exchangerate_$i"});
-      $form->{"exchangerate_$i"} ||= 1;
- 
+      $form->{"exchangerate_$i"} = $form->parse_amount($myconfig, $form->{"exchangerate_$i"}) || 1;
+      
+      # update exchangerate
+      $form->update_exchangerate($dbh, $form->{currency}, $form->{"datepaid_$i"}, $form->{"exchangerate_$i"});
+
       # record AR
       $amount = $form->round_amount($form->{"paid_$i"} * $form->{exchangerate}, $form->{precision});
 
@@ -1347,6 +1428,7 @@ sub post_invoice {
 
   
   # for dcn
+  $form->{oldinvtotal} ||= $invamount * $sw;
   ($form->{integer_amount}, $form->{decimal}) = split /\./, $form->{oldinvtotal};
   $form->{decimal} = substr("$form->{decimal}00", 0, 2);
 
@@ -1397,6 +1479,8 @@ sub post_invoice {
              |;
   $dbh->do($query) || $form->dberror($query);
 
+  $form->{invamount} = $invamount;
+
   # add shipto
   $form->{name} = $form->{customer};
   $form->{name} =~ s/--$form->{customer_id}//;
@@ -1405,6 +1489,9 @@ sub post_invoice {
   # save printed, emailed and queued
   $form->save_status($dbh);
 
+  # save references
+  $form->save_reference($dbh);
+
   # add link for order
   if ($form->{order_id}) {
     $query = qq|UPDATE oe SET aa_id = $form->{id}
@@ -1803,7 +1890,7 @@ sub reverse_invoice {
   $sth->finish;
   
   
-  for (qw(acc_trans dpt_trans invoice inventory shipto vr payment)) {
+  for (qw(acc_trans dpt_trans invoice inventory shipto vr payment reference)) {
     $query = qq|DELETE FROM $_ WHERE trans_id = $form->{id}|;
     $dbh->do($query) || $form->dberror($query);
   }
@@ -1821,13 +1908,13 @@ sub retrieve_invoice {
   my ($self, $myconfig, $form) = @_;
   
   # connect to database
-  my $dbh = $form->dbconnect_noauto($myconfig);
+  my $dbh = $form->dbconnect($myconfig);
 
   my $query;
   
   $form->{currencies} = $form->get_currencies($dbh, $myconfig);
  
-  if ($form->{id} *= 1) {
+  if ($form->{id}) {
     
     # retrieve invoice
     $query = qq|SELECT a.invnumber, a.ordnumber, a.quonumber,
@@ -1857,10 +1944,12 @@ sub retrieve_invoice {
     for (keys %$ref) { $form->{$_} = $ref->{$_} }
     $sth->finish;
 
+    $form->{payment_accno} = "";
     if ($form->{bank_accno}) {
       $form->{payment_accno} = ($form->{bank_accno_translation}) ? "$form->{bank_accno}--$form->{bank_accno_translation}" : "$form->{bank_accno}--$form->{bank_accno_description}";
     }
 
+    $form->{payment_method} = "";
     if ($form->{paymentmethod_id}) {
       $form->{payment_method} = "$form->{paymentmethod}--$form->{paymentmethod_id}";
     }
@@ -1886,7 +1975,8 @@ sub retrieve_invoice {
 		i.ponumber AS customerponumber, i.itemnotes, i.lineitemdetail,
 		p.partnumber, p.assembly, p.bin,
 		pg.partsgroup, p.partsgroup_id, p.partnumber AS sku,
-		p.listprice, p.lastcost, p.weight, p.onhand,
+		p.listprice, p.lastcost, i.fxsellprice AS sell, p.weight,
+		p.onhand,
 		p.inventory_accno_id, p.income_accno_id, p.expense_accno_id,
 		t.description AS partsgrouptranslation,
 		c.package, c.netweight, c.grossweight, c.volume
@@ -1936,9 +2026,11 @@ sub retrieve_invoice {
       $tth->finish;
       chop $ref->{taxaccounts};
 
+      $ref->{sellprice} = $form->round_amount($ref->{fxsellprice} * $form->{$form->{currency}}, $decimalplaces);
+      
       # price matrix
-      $ref->{sellprice} = ($ref->{fxsellprice} * $form->{$form->{currency}});
       &price_matrix($pmh, $ref, $form->{transdate}, $decimalplaces, $form, $myconfig);
+      # set original price
       $ref->{sellprice} = $ref->{fxsellprice};
 
       $ref->{partsgroup} = $ref->{partsgrouptranslation} if $ref->{partsgrouptranslation};
@@ -1951,11 +2043,8 @@ sub retrieve_invoice {
     $form->{transdate} = $form->current_date($myconfig);
   }
 
-  my $rc = $dbh->commit;
   $dbh->disconnect;
   
-  $rc;
-
 }
 
 
@@ -1966,7 +2055,7 @@ sub retrieve_item {
   my $dbh = $form->dbconnect($myconfig);
 
   my $i = $form->{rowcount};
-  my $null;
+  my $id;
   my $var;
 
   my $where = "WHERE p.obsolete = '0' AND NOT p.income_accno_id IS NULL";
@@ -1984,29 +2073,71 @@ sub retrieve_item {
     }
   }
 
-  if ($form->{"partsgroup_$i"} ne "") {
-    ($null, $var) = split /--/, $form->{"partsgroup_$i"};
-    $var *= 1;
-    if ($var == 0) {
-      # search by partsgroup, this is for the POS
-      $where .= qq| AND pg.partsgroup = '$form->{"partsgroup_$i"}'|;
+  my $query;
+  my $sth;
+  my $ref;
+  my $j;
+ 
+  if ($form->{"partsgroupcode_$i"} ne "") {
+    $var = $form->like(lc $form->{"partsgroupcode_$i"});
+    $query = qq|SELECT partsgroup, id FROM partsgroup
+                WHERE code LIKE '$var'|;
+    $sth = $dbh->prepare($query);
+    $sth->execute || $form->dberror($query);
+
+    $j = 0;
+    while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+      $j++;
+      $var = $ref->{partsgroup};
+      $id = $ref->{id};
+    }
+    $sth->finish;
+
+    if ($j) {
+      if ($j == 1) {
+	$form->{"partsgroup_$i"} = qq|$var--$id|;
+      } else {
+	$form->{"partsgroup_$i"} = "";
+	$var = $form->like(lc $form->{"partsgroupcode_$i"});
+	$where .= qq| AND pg.code LIKE '$var'|;
+      }
     } else {
-      $where .= qq| AND p.partsgroup_id = $var|;
+      $dbh->disconnect;
+      return 1;
     }
   }
 
+  if ($form->{"partsgroup_$i"} ne "") {
+    ($var, $id) = split /--/, $form->{"partsgroup_$i"};
+    $id *= 1;
+
+    if ($id) {
+      $where .= qq| AND (pg.partsgroup LIKE '${var}:%' OR p.partsgroup_id = $id)|;
+    } else {
+      $where .= qq| AND pg.partsgroup = '$var'|;
+
+      $form->{partsgroup} = $var;
+      $form->get_partsgroup($myconfig, { language_code => $form->{language_code}, searchitems => 'nolabor', pos => 1}, $dbh);
+      if (@{ $form->{all_partsgroup} }) {
+	$form->{rebuildpartsgroup} = 1;
+	$form->{parentgroups} = 0;
+      }
+    }
+  }
+  
   if ($form->{"description_$i"} ne "") {
     $where .= " ORDER BY 3";
   } else {
     $where .= " ORDER BY 2";
   }
   
-  my $query = qq|SELECT p.id, p.partnumber, p.description, p.sellprice,
-                 p.listprice, p.lastcost,
+  $query = qq|SELECT p.id, p.partnumber, p.description, p.sellprice,
+                 p.listprice, p.lastcost, p.sellprice AS sell,
 		 p.unit, p.assembly, p.bin, p.onhand, p.notes AS itemnotes,
 		 p.inventory_accno_id, p.income_accno_id, p.expense_accno_id,
-		 pg.partsgroup, p.partsgroup_id, p.partnumber AS sku,
-		 p.weight,
+		 pg.partsgroup, pg.code AS partsgroupcode, p.partsgroup_id,
+		 p.partnumber AS sku,
+		 p.weight, p.image,
 		 t1.description AS translation,
 		 t2.description AS grouptranslation
                  FROM parts p
@@ -2014,10 +2145,9 @@ sub retrieve_item {
 		 LEFT JOIN translation t1 ON (t1.trans_id = p.id AND t1.language_code = '$form->{language_code}')
 		 LEFT JOIN translation t2 ON (t2.trans_id = p.partsgroup_id AND t2.language_code = '$form->{language_code}')
 	         $where|;
-  my $sth = $dbh->prepare($query);
+  $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
-  my $ref;
   my $ptref;
 
   # setup exchange rates
@@ -2064,6 +2194,8 @@ sub retrieve_item {
   
   $sth->finish;
   $dbh->disconnect;
+
+  1;
   
 }
 
@@ -2192,13 +2324,13 @@ sub exchangerate_defaults {
   $form->{currencies} = $form->get_currencies($dbh, $myconfig);
   $form->{defaultcurrency} = substr($form->{currencies},0,3);
   
-  $query = qq|SELECT buy
+  $query = qq|SELECT exchangerate
               FROM exchangerate
 	      WHERE curr = ?
 	      AND transdate = ?|;
   my $eth1 = $dbh->prepare($query) || $form->dberror($query);
 
-  $query = qq~SELECT max(transdate || ' ' || buy || ' ' || curr)
+  $query = qq~SELECT max(transdate || ' ' || exchangerate || ' ' || curr)
               FROM exchangerate
 	      WHERE curr = ?~;
   my $eth2 = $dbh->prepare($query) || $form->dberror($query);
@@ -2218,12 +2350,90 @@ sub exchangerate_defaults {
     $eth1->finish;
   }
 
-  $form->{$form->{currency}} = $form->{exchangerate} if $form->{exchangerate};
-  $form->{$form->{currency}} ||= 1;
+  $form->{$form->{currency}} = $form->{exchangerate} || 1;
   $form->{$form->{defaultcurrency}} = 1;
 
 }
 
 
+sub generate_invoice {
+  my ($self, $myconfig, $form) = @_;
+  
+  # connect to database, turn off autocommit
+  my $dbh = $form->dbconnect_noauto($myconfig);
+
+  my $query;
+  my $sth;
+  my $ref;
+
+
+  if ($form->{employee}) {
+    $query = qq|SELECT vc.name AS $form->{vc},
+                ad.city
+		FROM $form->{vc} vc
+		JOIN address ad ON (ad.trans_id = vc.id)
+		WHERE vc.id = $form->{"$form->{vc}_id"}|;
+  } else {
+    $query = qq|SELECT vc.name AS $form->{vc},
+                ad.city,
+                e.name AS employee, vc.employee_id
+		FROM $form->{vc} vc
+		LEFT JOIN employee e ON (e.id = vc.employee_id)
+		JOIN address ad ON (ad.trans_id = vc.id)
+		WHERE vc.id = $form->{"$form->{vc}_id"}|;
+  }
+  $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  $ref = $sth->fetchrow_hashref(NAME_lc);
+  for (keys %$ref) { $form->{$_} = $ref->{$_} }
+  $sth->finish;
+
+  $query = qq|SELECT c.accno
+              FROM chart c
+	      JOIN $form->{vc}tax ct ON (ct.chart_id = c.id)
+	      WHERE ct.$form->{vc}_id = $form->{"$form->{vc}_id"}|;
+  $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  $form->{taxaccounts} = "";
+  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+    $form->{taxaccounts} .= "$ref->{accno} ";
+  }
+  $sth->finish;
+  chop $form->{taxaccounts};
+
+  # tax rates
+  $query = qq|SELECT c.accno, c.description, t.rate, t.taxnumber,
+              l.description AS translation
+	      FROM chart c
+	      JOIN tax t ON (c.id = t.chart_id)
+	      LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$form->{language_code}')
+	      WHERE c.link LIKE '%AR_tax%'
+	      AND (t.validto >= '$form->{transdate}' OR t.validto IS NULL)
+	      ORDER BY accno, validto|;
+  $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  my %tax = ();
+  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+    if (not exists $tax{$ref->{accno}}) {
+      $ref->{description} = $ref->{translation} if $ref->{translation};
+      for (qw(rate description taxnumber)) { $form->{"$ref->{accno}_$_"} = $ref->{$_} }
+      $tax{$ref->{accno}} = 1;
+    }
+  }
+  $sth->finish;
+  
+
+  my $rc = IS->post_invoice($myconfig, $form, $dbh);
+
+  $dbh->disconnect;
+
+  $rc;
+
+}
+
+
 1;
 
